define :cloudformation_update, :stack => nil,
       :access_key => nil, :secret_key => nil, :region => nil, :output => nil,
       :delete_bucket_on_create => nil do

  # Defines variables
  name = params[:name]
  stack = params[:stack]
  unless stack
    stack = name
  end
  access_key = params['access_key']
  unless access_key
    access_key = node['alphard']['aws']['access_key']
  end
  secret_key = params['secret_key']
  unless secret_key
    secret_key = node['alphard']['aws']['secret_key']
  end
  region = params['region']
  unless region
    region = node['alphard']['aws']['region']
  end
  output = params['output']
  unless output
    output = node['alphard']['aws']['output']
  end
  delete_buckets_on_create = params['delete_buckets_on_create']
  unless delete_buckets_on_create
    delete_buckets_on_create = node['alphard']['aws']['cloudformation']['delete_buckets_on_create']
  end
  user = node['alphard']['aws']['user']
  group = node['alphard']['aws']['group']
  template_directory = "#{Chef::Config[:file_cache_path]}/cloudformation"
  template_source_file = node['alphard']['aws']['cloudformation']['stacks']['default']['template']
  if node['alphard']['aws']['cloudformation']['stacks'][stack]
    template_source_file = node['alphard']['aws']['cloudformation']['stacks'][stack]['template']
  end
  template_file = "#{template_directory}/#{stack}.template"
  node.default[:'alphard']['aws']['cloudformation']['stacks'][stack]['action'] = 'CREATE'

  # Creates template directory
  directory template_directory do
    user user
    group group
    mode 00755
    recursive true
    action :create
  end

  # Creates of updates template and notifies
  if template_source_file.end_with?('.erb')
    template template_file do
      source template_source_file
      user user
      group group
      mode 00644
      notifies :run, 'ruby_block[manage-stacks]'
      action :create
    end
  else
    cookbook_file template_file do
      source template_source_file
      user user
      group group
      mode 00644
      notifies :run, 'ruby_block[manage-stacks]'
      action :create
    end
  end

  # Lists stacks
  # CREATE_IN_PROGRESS => NOTHING
  # CREATE_FAILED => NOTHING
  # CREATE_COMPLETE => UPDATE
  # ROLLBACK_IN_PROGRESS => NOTHING
  # ROLLBACK_FAILED => NOTHING
  # ROLLBACK_COMPLETE => NOTHING
  # DELETE_IN_PROGRESS => NOTHING
  # DELETE_FAILED => NOTHING
  # DELETE_COMPLETE => CREATE
  # UPDATE_IN_PROGRESS => NOTHING
  # UPDATE_COMPLETE_CLEANUP_IN_PROGRESS => NOTHING
  # UPDATE_COMPLETE => UPDATE
  # UPDATE_ROLLBACK_IN_PROGRESS => NOTHING
  # UPDATE_ROLLBACK_FAILED => NOTHING
  # UPDATE_ROLLBACK_COMPLETE_CLEANUP_IN_PROGRESS => NOTHING
  # UPDATE_ROLLBACK_COMPLETE => UPDATE
  ruby_block 'manage-stacks' do
    # noinspection RubyScope
    block do
      require 'colorize'
      require 'json'

      ENV['AWS_ACCESS_KEY_ID'] = access_key
      ENV['AWS_SECRET_ACCESS_KEY'] = secret_key
      ENV['AWS_DEFAULT_REGION'] = region
      ENV['AWS_DEFAULT_OUTPUT']= output

      def aws(command)
        full_command = "aws #{command}"
        Chef::Log.info "execute: #{full_command}"
        result = %x(#{full_command} 2>&1).strip
        exit_code = $?.exitstatus
        if exit_code > 0
          raise result
        else
          return result
        end
      end

      def delete_objects(bucket)
        def ls(bucket)
          json = aws("s3api list-object-versions --bucket #{bucket}")
          out = JSON.parse(json)
          truncated = out['IsTruncated']
          objects = (out['DeleteMarkers'] + out['Versions']).map { |entry|
            {:Key => entry['Key'], :VersionId => entry['VersionId']}
          }
          return truncated, objects
        end

        def del(bucket, objects)
          if objects.length != 0
            json = JSON.generate({:Objects => objects, :Quiet => false})
            aws("s3api delete-objects --bucket #{bucket} --delete '#{json}'")
          end
        end

        truncated = true
        while truncated do
          truncated, objects = ls(bucket)
          del(bucket, objects)
        end
      end

      begin

        # Defines actions CREATE | UPDATE | DELETE
        action = 'CREATE'

        stacks = aws('cloudformation list-stacks')
        stacks = JSON.parse(stacks.strip)
        stacks['StackSummaries'].each { |stack_summary|
          name = stack_summary['StackName']
          status = stack_summary['StackStatus']
          if name == stack
            if status.include?('IN_PROGRESS') || status.include?('FAILED')
              action = 'NOTHING'
              puts "\n\n    - invalid stack status: #{status}\n".yellow
            elsif status == 'CREATE_COMPLETE' || status == 'UPDATE_COMPLETE' || status == 'UPDATE_ROLLBACK_COMPLETE'
              action = 'UPDATE'
            end
          end
        }

        if action == 'CREATE'
          # Deletes previous s3 buckets
          if delete_buckets_on_create
            tags = {'aws:cloudformation:stack-name' => stack}
            bucket_list = aws('s3 ls')
            bucket_list.lines.each { |line|
              begin
                bucket = line.split(' ')[2]
                bucket_tags = aws("s3api get-bucket-tagging --bucket #{bucket}")
                bucket_tags_set = JSON.parse(bucket_tags)['TagSet']
                if tags && bucket_tags_set
                  bucket_delete = false
                  bucket_tags_set.each { |tag|
                    bucket_tag_key = tag['Key']
                    bucket_tag_value = tag['Value']
                    bucket_delete = tags[bucket_tag_key] == bucket_tag_value
                    if bucket_delete
                      break
                    end
                  }
                  if bucket_delete
                    puts "\n\n    - delete bucket #{bucket}\n".green
                    delete_objects(bucket)
                    aws("s3 rb s3://#{bucket}")
                  end
                end
              rescue Exception => _
              end
            }
          end

          # Creates stack
          puts "\n\n    - create stack #{stack}\n".green
          aws("cloudformation create-stack --capabilities CAPABILITY_IAM --stack-name #{stack} --template-body file://#{template_file}")
        elsif action == 'UPDATE'
          # Updates stack
          puts "\n\n    - update stack #{stack}\n".green
          begin
            aws("cloudformation update-stack --capabilities CAPABILITY_IAM --stack-name #{stack} --template-body file://#{template_file}")
          rescue Exception => e
            puts "\n\n    - no update: #{e.message}\n".yellow
          end
        end
      rescue Exception => e
        Chef::Log.error e.message.red
      end
    end
    action :nothing
  end

end