#
# Author:: Frédéric Nowak (<frederic.nowak@alphard.io>)
# Cookbook Name:: alphard-aws
# file:: cloudformation
#
# Copyright 2011, Hydra Technologies, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require 'aws-sdk'

Ohai.plugin(:Cloudformation) do
  provides 'cloudformation'

  depends 'ec2'
  depends 'ec2_region'
  depends 'ec2_tags'

  collect_data do

    if ec2 != nil && ec2_region != nil && ec2_tags != nil
      Ohai::Log.debug('ec2_region and ec2_tags hints are presents.')
      begin
        cloudformation Mash.new()
        cloudformation[:logical_id] = ec2_tags['aws:cloudformation:logical-id']
        cloudformation[:stack_id] = ec2_tags['aws:cloudformation:stack-id']
        cloudformation[:stack_name] = ec2_tags['aws:cloudformation:stack-name']
        cloudformation[:stack_resources] = Mash.new()

        cloudformation_client = Aws::CloudFormation::Client.new(region: ec2_region)
        cloudformation_client.describe_stack_resources({ stack_name: cloudformation[:stack_name] }).stack_resources.each { |resource|
          cloudformation_resource = Mash.new()
          cloudformation_resource[:type] = resource.resource_type
          cloudformation_resource[:logical_id] = resource.logical_resource_id
          cloudformation_resource[:physical_id] = resource.physical_resource_id
          cloudformation_resource[:status] = resource.resource_status
          cloudformation_resource[:status_reason] = resource.resource_status_reason
          cloudformation_resource[:timestamp] = resource.timestamp
          cloudformation_resource[:description] = resource.description
          cloudformation[:stack_resources][resource.logical_resource_id] = cloudformation_resource
        }
      rescue Exception => exception
        Ohai::Log.warn('cloudformation exception: ' + exception.to_s)
        false
      end
    else
      Ohai::Log.debug('No hints present for ec2_region and ec2_tags.')
      false
    end

  end

end