#
# Author:: Frédéric Nowak (<frederic.nowak@alphard.io>)
# Cookbook Name:: alphard-aws
# Attribute:: ec2_tags
#
# Copyright 2011, Hydra Technologies, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require 'aws-sdk'

Ohai.plugin(:Ec2Tags) do
  provides 'ec2_tags'

  depends 'ec2'
  depends 'ec2_region'

  collect_data do
    if ec2 != nil && ec2_region != nil
      Ohai::Log.debug('ec2 and ec2_region hints are presents.')
      begin
        ec2_tags Mash.new()
        ec2_client = Aws::EC2::Client.new(region: ec2_region)
        ec2_instance_filter = {:name => 'resource-type', :values => ['instance']}
        ec2_client.describe_tags(filters: [ec2_instance_filter]).tags.each { |tag|
          ec2_tags[tag.key] = tag.value
        }
      rescue Exception => exception
        Ohai::Log.warn('ec2_tags exception: ' + exception.to_s)
        false
      end
    else
      Ohai::Log.debug('No hints present for ec2. No instance tags computed.')
      false
    end
  end

end