provides :aws

def whyrun_supported?
  true
end

action :run do

  # Defines variables
  resource = @new_resource
  name = resource.name
  command = resource.command
  unless command
    command = name
  end
  access_key = resource.access_key
  unless access_key
    access_key = node['alphard']['aws']['access_key']
  end
  secret_key = resource.secret_key
  unless secret_key
    secret_key = node['alphard']['aws']['secret_key']
  end
  region = resource.region
  unless region
    region = node['alphard']['aws']['region']
  end
  output = resource.output
  unless output
    output = node['alphard']['aws']['output']
  end
  verbose = resource.verbose
  unless verbose
    verbose = node['alphard']['aws']['verbose']
  end
  ignore_failure = resource.ignore_failure
  callback = resource.callback

  # Includes awscli
  require 'colorize'

  # Configures awscli
  ENV['AWS_ACCESS_KEY_ID'] = access_key
  ENV['AWS_SECRET_ACCESS_KEY'] = secret_key
  ENV['AWS_DEFAULT_REGION'] = region
  ENV['AWS_DEFAULT_OUTPUT']=output

  if verbose && (ENV['AWS_ACCESS_KEY_ID'] == nil || ENV['AWS_SECRET_ACCESS_KEY'] == nil)
    Chef::Log.warn 'nor access keys or secret keys provided!'.yellow
  end

  # Executes awscli
  begin
    system_command = "aws #{command}"
    result = nil
    converge_by("execute command [#{region}/#{output}]: #{system_command}\n") do
      result = %x(#{system_command} 2>&1).strip
    end
    exit_code = $?.exitstatus
    if exit_code > 0
      raise result
    else
      if verbose
        puts "\n      #{result.gsub("\n", "\n      ").colorize(:blue)}"
      end
      callback.call(result)
    end
  rescue Exception => e
    if verbose && ignore_failure
      Chef::Log.warn e.message.yellow
    else
      raise e
    end
  end

end